"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:libbasic2' }"

Time subclass:#TimeDuration
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Magnitude-General'
!

!TimeDuration class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Instances of time represent a particular time-of-day.
    Since they only store hours, minutes and seconds within a day,
    they cannot be used to compare times across midnight 
    (i.e. they should not be used as timeStamps).

    Use instances of AbsoluteTime (and read the comment there) to do this.

    Examples:
        |t|

        t := Time now.
        Transcript showCR:t.


        |t1 t2|

        t1 := Time now.
        (Delay forSeconds:10) wait.
        t2 := Time now.
        t2 - t1   

    [author:]
        Claus Gittinger

    [see also:]
        Date AbsoluteTime AbstractTime OperatingSystem
        Filename
"
! !

!TimeDuration class methodsFor:'instance creation'!

hours:h minutes:m seconds:s millis:millis
    "return an instance of Time representing the given time.
     See also Time now / Date today / AbsoluteTime now."

    ^ self basicNew setHours:h minutes:m seconds:s millis:millis

    "
     TimeDuration hours:2 minutes:33 seconds:0 millis:123
    "
! !

!TimeDuration class methodsFor:'format strings'!

formatString12us
    "return the format string used to format US times (and other areas)"

    ^ '%u:%m:%s.%i %a'
!

formatString24
    "return the format string used to format european times (and other areas)"

    ^ '%h:%m:%s.%i'
! !

!TimeDuration methodsFor:'accessing'!

milliseconds
    ^ timeEncoding \\ 1000
! !

!TimeDuration methodsFor:'converting'!

asTime
    "return a Time object from the receiver - thats the receiver."

    ^ Time hours:(self hours) minutes:(self minutes) seconds:(self seconds)
! !

!TimeDuration methodsFor:'private'!

getMilliseconds
    "return the number of milliseconds since midnight"

    ^ timeEncoding
!

getSeconds
    "return the number of seconds since midnight"

    ^ timeEncoding // 1000
!

setHours:h minutes:m seconds:s millis:millis
    "set my time given individual values"

    self setMilliseconds:(((h\\24) * 60 * 60 ) + (m * 60) + s) * 1000 + millis.
!

setMilliseconds:millis
    "set my time given milliseconds since midnight"

    timeEncoding := millis
!

setSeconds:secs
    "set my time given seconds since midnight"

    self setMilliseconds:(secs * 1000).
! !

!TimeDuration class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic2/Attic/TimeDuration.st,v 1.1 2001-11-08 16:03:35 md Exp $'
! !
